@@ -18,6 +18,7 @@
  *
  * See the file COPYING included with this distribution for more details.
  */
+#include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
@@ -54,6 +55,8 @@ static void iscsid_startup(void)
 	if (system(startup_cmd) < 0)
 		log_error("Could not execute '%s' (err %d)",
 			  startup_cmd, errno);
+
+	free(startup_cmd);
 }
 
 #define MAXSLEEP 128
@@ -94,13 +97,25 @@ static int ipc_connect(int *fd, char *unix_sock_name, int start_iscsid)
 		if (nsec <= MAXSLEEP/2)
 			sleep(nsec);
 	}
+	close(*fd);
+	*fd = -1;
 	log_error("can not connect to iSCSI daemon (%d)!", errno);
 	return ISCSI_ERR_ISCSID_NOTCONN;
 }
 
+char iscsid_namespace[64] = ISCSIADM_NAMESPACE;
+
+void iscsid_set_namespace(pid_t pid) {
+	if (pid) {
+		snprintf(iscsid_namespace, 64, ISCSIADM_NAMESPACE "-%d", pid);
+	} else {
+		snprintf(iscsid_namespace, 64, ISCSIADM_NAMESPACE);
+	}
+}
+
 static int iscsid_connect(int *fd, int start_iscsid)
 {
-	return ipc_connect(fd, ISCSIADM_NAMESPACE, start_iscsid);
+	return ipc_connect(fd, iscsid_namespace, start_iscsid);
 }
 
 int iscsid_request(int *fd, iscsiadm_req_t *req, int start_iscsid)
@@ -141,7 +156,7 @@ int iscsid_response(int fd, iscsiadm_cmd_e cmd, iscsiadm_rsp_t *rsp,
 		if (!err) {
 			if (poll_wait)
 				continue;
-			return ISCSI_ERR_ISCSID_NOTCONN;
+			return ISCSI_ERR_SESSION_NOT_CONNECTED;
 		} else if (err < 0) {
 			if (errno == EINTR)
 				continue;
@@ -241,7 +256,8 @@ int uip_broadcast(void *buf, size_t buf_len, int fd_flags, uint32_t *status)
 	iscsid_uip_rsp_t rsp;
 	int flags;
 	int count;
-
+	size_t write_res;
+	
 	err = uip_connect(&fd);
 	if (err) {
 		log_warning("uIP daemon is not up");
@@ -251,10 +267,10 @@ int uip_broadcast(void *buf, size_t buf_len, int fd_flags, uint32_t *status)
 	log_debug(3, "connected to uIP daemon");
 
 	/*  Send the data to uIP */
-	err = write(fd, buf, buf_len);
-	if (err != buf_len) {
+	write_res = write(fd, buf, buf_len);
+	if (write_res != buf_len) {
 		log_error("got write error (%d/%d), daemon died?",
-			  err, errno);
+			  (int)write_res, errno);
 		close(fd);
 		return ISCSI_ERR_ISCSID_COMM_ERR;
 	}
@@ -283,9 +299,8 @@ int uip_broadcast(void *buf, size_t buf_len, int fd_flags, uint32_t *status)
 		/*  Wait for the response */
 		err = read(fd, &rsp, sizeof(rsp));
 		if (err == sizeof(rsp)) {
-			log_debug(3, "Broadcasted to uIP with length: %ld "
-				     "cmd: 0x%x rsp: 0x%x", buf_len,
-				     rsp.command, rsp.err);
+			log_debug(3, "Broadcasted to uIP with length: %zu cmd: 0x%x rsp: 0x%x",
+				  buf_len, rsp.command, rsp.err);
 			err = 0;
 			break;
 		} else if ((err == -1) && (errno == EAGAIN)) {
